/********************************************************************
*
* File: Wtree.c - Build GUI to display the widget hierarchy in.
*
*    This file was originally generated by the program lxb.
*
*	Copyright 1997  By Bruce Parkin
*
********************************************************************/

/* include files */
#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>    
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <X11/Xmu/Editres.h> 
#include <X11/Xaw/Tree.h>
#include <X11/Xaw/Panner.h>
#include <X11/Xaw/Porthole.h>

#include "lxb.h"
#include "Wtree.h"

/* global */
Widget tree;

/* local prototypes */
static void
PortholeCallback(Widget w, XtPointer panner_ptr, XtPointer report_ptr);
static void
PannerCallback(Widget w, XtPointer closure, XtPointer report_ptr);

/**************************************************
*
* Function: build_wtree_area
* 
* 	Build the Widget hierarchy display area
*
*	Inputs:
*		Parent: the main area form
*		right_child: an optional widget to place in the main form
* 
* Returns:
*		toplevel widget of area
* 
**************************************************/
Widget
build_wtree_area(Widget parent, Widget *right_child)
{
	int n;
  Arg args[20];
  Widget form_4, form_1, form_2;
	Widget pushButton_6;
  Widget frame_6, frame_7;
  Widget panner, porthole;
  XmString xmstrings[10];

  /* overall parent */
  form_1 = XtCreateManagedWidget("wtree", xmFormWidgetClass, parent, args, 0);

  /* top form */
  XtSetArg(args[0], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_OPPOSITE_FORM);
  XtSetArg(args[4], XmNbottomOffset, -75);
  form_2 = XtCreateManagedWidget("form_2", xmFormWidgetClass, form_1, args, 5);

  /* form to put panner widget in */
  XtSetArg(args[0], XmNrightAttachment, XmATTACH_OPPOSITE_FORM);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNrightOffset, -75);
  form_4 = XtCreateManagedWidget("form_1", xmFormWidgetClass, form_2, args, 5);

  /* put the panner widget inside */
  XtSetArg(args[0], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNshadowType, XmSHADOW_OUT);
  XtSetArg(args[5], XmNshadowThickness, 3);
  frame_6 = XtCreateManagedWidget("frame_6", xmFrameWidgetClass, form_4, args, 6);
                                                                                         
  XtSetArg(args[0], XtNresize, False);
  panner = XtCreateManagedWidget("panner", pannerWidgetClass, frame_6, args, 1);

	/* area to place right_child in */
  XtSetArg(args[0], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_WIDGET);
  XtSetArg(args[2], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNleftWidget, form_4);
  *right_child = XtCreateManagedWidget("right_child", xmFormWidgetClass, form_2, args, 5);

  /* a frame to contain the bottom porthole and tree */
  XtSetArg(args[0], XmNtopWidget, form_2);
  XtSetArg(args[1], XmNrightAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[3], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[5], XmNshadowType, XmSHADOW_OUT);
  XtSetArg(args[6], XmNshadowThickness, 3);
  frame_7 = XtCreateManagedWidget("frame_7", xmFrameWidgetClass, form_1, args, 7);

  porthole = XtCreateManagedWidget("porthole", portholeWidgetClass, frame_7, args, 0);
  tree = XtCreateManagedWidget("tree", treeWidgetClass, porthole, args, 0);

  XtAddCallback(porthole, XtNreportCallback, PortholeCallback, (XtPointer) panner);
  XtAddCallback(panner, XtNreportCallback, PannerCallback, (XtPointer) porthole);
             
	return (form_1);
}
/**********************************************************************
*
*  Function Name: PortholeCallback
*
*  Description: called when the porthole or its child has changed
*
*  Arguments: porthole - the porthole widget.
*             panner_ptr - the panner widget.
*             report_ptr - the porthole record.
*
*  Returns: none.
*
**********************************************************************/
static void
PortholeCallback(Widget w, XtPointer panner_ptr, XtPointer report_ptr)
{
    Arg args[10];
    Cardinal n = 0;
    XawPannerReport *report = (XawPannerReport *) report_ptr;
    Widget panner = (Widget) panner_ptr;
 
    XtSetArg (args[n], XtNsliderX, report->slider_x); n++;
    XtSetArg (args[n], XtNsliderY, report->slider_y); n++;

    if (report->changed != (XawPRSliderX | XawPRSliderY)) {
			XtSetArg (args[n], XtNsliderWidth, report->slider_width); n++;
			XtSetArg (args[n], XtNsliderHeight, report->slider_height); n++;
			XtSetArg (args[n], XtNcanvasWidth, report->canvas_width); n++;
			XtSetArg (args[n], XtNcanvasHeight, report->canvas_height); n++;
    };

    XtSetValues (panner, args, n);
}
/**********************************************************************
*
*  Function Name: PannerCallback
*
*  Description: called when the panner has moved.
*
*  Arguments: panner - the panner widget.
*             closure - *** NOT USED ***.
*             report_ptr - the panner record.
*
*  Returns: none.
*
**********************************************************************/
static void
PannerCallback(Widget w, XtPointer closure, XtPointer report_ptr)
{
    Arg args[2];
    XawPannerReport *report = (XawPannerReport *) report_ptr;
 
    XtSetArg (args[0], XtNx, -report->slider_x);
    XtSetArg (args[1], XtNy, -report->slider_y);
 
    XtSetValues(tree, args, 2);
}
