/**************************************************
 * File: code.c
 *
 *   Copyright (C) 1995  Bruce Parkin 
 *
 **************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>    
#include <errno.h>
/* makedepend does not like this on linux
#include <ctype.h>    
*/

#ifdef SOLARIS24
#include <string.h>    
#else
#include <strings.h>    
#endif

#include <Xm/Xm.h>

#include "lxb.h"
#include "top.h"
#include "code_util.h"
#include "code_gui.h"
#include "code_xdefs.h"
#include "code_hdr.h"
#include "code_special.h"
#include "code.h"

/* local prototypes */
static int
make_makefile(char *appname);

static int
make_appheader(char *appname);

static int
make_appcfile(char *appname);

static int
make_callfile(char *appname);

static int
make_callheader(char *appname);

/**************************************************
 *
 * Function: 
 *
 **************************************************/
static int
make_callfile(char *appname)
{
}
/**************************************************
 *
 * Function: make_callheader - code the callbacks
 *	header file.
 *
 **************************************************/
static int
make_callheader(char *appname)
{
	int 	status;
	FILE	*file;
	char	all[70];
	WSTRUCT *ptr;
	CALL *cptr;

	strcpy(all, appname);
	strcat(all, "-callbacks.h");

	/* don't overwrite the old file */
	status = check_file(all);
	if (status != OK)
		return (NOTOK);

	/* create the header file*/
  file = fopen(all, "w");
  if (file == NULL){
    perror("Problem with c header file");
    return (NOTOK);
  };

	fprintf(file, "/******************************************************************\n");
	fprintf(file, "*\n");
	fprintf(file, "* File: %s-callbacks.h - callbacks source file\n", appname);
	fprintf(file, "*\n");
	fprintf(file, "*    This file is generated by the program lxb.\n");
	fprintf(file, "*\n");
	fprintf(file, "*******************************************************************/\n\n");


	ptr = top_ptr;
	while (ptr != NULL){

		cptr = ptr->callptr;
		while (cptr != NULL) {


			cptr = cptr->next;
		};
		ptr = ptr->last;
	};

	fprintf(file, "\n");

	fclose(file);

	return (OK);
}
/**************************************************
 *
 * Function: check_file
 *
 **************************************************/
int
check_file(char *filename)
{
  int file;

  /* don't overwrite the old file */
  file = open(filename, O_CREAT|O_EXCL, 0644);
  if (file == EEXIST){

    fprintf(stderr, "File %s exists.\n", filename);
    fprintf(stderr, "First delete the file then try again.\n");
    return (NOTOK);

  } else if (file == -1){
    fprintf(stderr, "File: %s\n", filename);
    perror("Attempt to write application file failed");
    return (NOTOK);

  } else{
    close (file);
  };

  return (OK);
}                               
/**************************************************
 *
 * Function: make_appcfile
 *
 **************************************************/
static int
make_appcfile(char *appname)
{
	int 	status;
	FILE	*file;
	char	all[70];

	strcpy(all, appname);
	strcat(all, ".c");

	/* don't overwrite the old file */
	status = check_file(all);
	if (status != OK)
		return (NOTOK);

	/* create the c source file*/
  file = fopen(all, "w");
  if (file == NULL){
    perror("Problem with c source file");
    return (NOTOK);
  };

	fprintf(file, "/**************************************************\n");
	fprintf(file, "* File: %s.c - application source file\n", appname);
	fprintf(file, "*\n");
	fprintf(file, "*    This file is generated by the program lxb.\n");
	fprintf(file, "*\n");
	fprintf(file, "**************************************************/\n\n");
	fprintf(file, "/* include files */\n");
	fprintf(file, "#include <stdio.h>\n\n");
	fprintf(file, "#include <Xm/Xm.h>\n");
	fprintf(file, "#include <Xm/ScrolledW.h>\n");
	fprintf(file, "#include <Xm/ScrollBar.h>\n");
	fprintf(file, "#include <Xm/PushB.h>\n");
	fprintf(file, "#include <Xm/ToggleB.h>\n");
	fprintf(file, "#include <Xm/ArrowB.h>\n");
	fprintf(file, "#include <Xm/CascadeB.h>\n");
	fprintf(file, "#include <Xm/Separator.h>\n");
	fprintf(file, "#include <Xm/DrawnB.h>\n");
	fprintf(file, "#include <Xm/Scale.h>\n");
	fprintf(file, "#include <Xm/Frame.h>\n");
	fprintf(file, "#include <Xm/Form.h>\n");
	fprintf(file, "#include <Xm/RowColumn.h>\n");
	fprintf(file, "#include <Xm/Label.h>\n");
	fprintf(file, "#include <Xm/TextF.h>\n");
	fprintf(file, "#include <Xm/Text.h>\n");
	fprintf(file, "#include <Xm/List.h>\n");
	fprintf(file, "#include <Xm/DrawingA.h>\n");
	fprintf(file, "#include <Xm/MenuShell.h>\n");
	fprintf(file, "#include <X11/Shell.h>\n\n");
	fprintf(file, "#include \"%s_util.h\"\n", appname);
	fprintf(file, "#include \"%s.h\"\n\n", appname);

	/* dec/def global vars here */
	status = code_global_vars(file);
	if (status != OK){
		perror("Problems with writing X code (global variables)");
		fclose(file);
		return (NOTOK);
	};

	fprintf(file, "/* local prototypes */\n");
	fprintf(file, "static void\n");
	fprintf(file, "build_gui(Widget %s);\n\n", top_ptr->appname);
	fprintf(file, "/**************************************************\n");
	fprintf(file, "* Function: build_gui\n");
	fprintf(file, "**************************************************/\n");
	fprintf(file, "static void\n");
	fprintf(file, "build_gui(Widget %s)\n", top_ptr->appname);
	fprintf(file, "{\n");
	fprintf(file, "\tArg args[%d];\n", MAX_ARGS);
	fprintf(file, "\tXmString xmstrings[%d];\n", MAX_XMSTRINGS);
	fprintf(file, "\tXmFontList fontlist;\n");

	status = code_local_vars(file);
	if (status != OK){
		perror("Problems with writing X code (local variables)");
		fclose(file);
		return (NOTOK);
	};

	/* create widget and set resources code */
	status = code_gui(file);
	if (status != OK){
		perror("Problems with writing X code");
		fclose(file);
		return (NOTOK);
	};

	fprintf(file, "}\n");

	fprintf(file, "/**************************************************\n");
	fprintf(file, "* Function: main\n");
	fprintf(file, "**************************************************/\n");
	fprintf(file, "int \n");
	fprintf(file, "main(int argc, char *argv[])\n");
	fprintf(file, "{\n");

	fprintf(file, "\tArg args[%d];\n\n", MAX_ARGS);
	fprintf(file, "\t/* Initialize toolkit and parse command line options. */\n");

	fprintf(file, "\t%s = XtInitialize(NULL, \"%s\", NULL, 0, &argc, argv);\n", 
						top_ptr->appname, appname); 

	fprintf(file, "\n");

/*******************/
{
	RES *rptr;

  /* One fallback Shell resource - geometry */
	rptr = top_ptr->resptr;

	/* normal resource */
	fprintf(file, "\tXtSetArg(args[0], XmN%s, \"%s\");\n", rptr->name, rptr->value);
	fprintf(file, "\tXtSetValues(%s, args, 1);\n", top_ptr->appname); 
}                                 

/*******************/

	fprintf(file, "\n");

	fprintf(file, "\tbuild_gui(%s);\n\n", top_ptr->appname);
	fprintf(file, "\tXtRealizeWidget(%s);\n\n", top_ptr->appname);
	fprintf(file, "\tXtMainLoop();\n");
	fprintf(file, "}\n");

	fclose(file);

	return (OK);
}
/**************************************************
 *
 * Function: make_appheader
 *
 **************************************************/
static int
make_appheader(char *appname)
{
	int 	status;
	FILE	*file;
	char	all[70];

	strcpy(all, appname);
	strcat(all, ".h");

	/* don't overwrite the old file */
	status = check_file(all);
	if (status != OK)
		return (NOTOK);

	/* create the header file*/
  file = fopen(all, "w");
  if (file == NULL){
    perror("Problem with c header file");
    return (NOTOK);
  };

	fprintf(file, "/**************************************************\n");
	fprintf(file, "* File: %s.h - application source file\n", appname);
	fprintf(file, "*\n");
	fprintf(file, "*    This file is generated by the program lxb.\n");
	fprintf(file, "*\n");
	fprintf(file, "**************************************************/\n\n");

	code_header_file(file);

	fclose(file);

	return (OK);
}
/**************************************************
 *
 * Function: make_makefile
 *
 **************************************************/
static int
make_makefile(char *appname)
{
	int 		status;
	FILE		*file;
	char 		*xversion;
	Boolean	x11r5;

	char	all[100];

	strcpy(all, appname);
	strcat(all, "-makefile");

	/* don't overwrite the old file */
	status = check_file(all);
	if (status != OK)
		return (NOTOK);

	/* create the makefile */
  file = fopen(all, "w");
  if (file == NULL){
    perror("problem with Makefile file");
    return (NOTOK);
  };

	/* hack for Motif and X versions */
	x11r5 = False;
	xversion = getenv("XVERSION");
	if (xversion != NULL){
		if ( (strcmp(xversion, "X11R5") == 0) ){
			x11r5 = True;
		};
	};

	fprintf(file, "#---------------------------------------------------\n");
	fprintf(file, "# File: Makefile for ");
	fprintf(file, "%s\n", appname);
	fprintf(file, "#\n");
	fprintf(file, "#    This file is generated by the program lxb.\n");
	fprintf(file, "#\n");
	fprintf(file, "#---------------------------------------------------\n\n");
	fprintf(file, "APPNAME = %s\n", appname);
	fprintf(file, "UTILNAME = %s_util\n", appname);
	fprintf(file, "PROGRAM = $(APPNAME)\n\n");
	fprintf(file, "DEFINES = \n");
	fprintf(file, "CDEBUGFLAGS = -g\n");
	fprintf(file, "INCLUDES = -I.\n");
	fprintf(file, "CFLAGS = $(CDEBUGFLAGS) $(INCLUDES) $(DEFINES)\n\n");

	if (x11r5)
		fprintf(file, "XVERSION = X11R5\n\n");
	else
		fprintf(file, "XVERSION = X11R6\n\n");

	if (x11r5)
		fprintf(file, "XLIBS = -lXm -L/usr/X11R5/lib -lXt -lXmu -lX11\n");
	else
		fprintf(file, "XLIBS = -L/usr/X11/lib -lXm -lXpm -lXt -lXext -lX11\n");

	fprintf(file, "LDFLAGS = -o\n\n");
	fprintf(file, "OBJS = 	$(APPNAME).o $(UTILNAME).o\n\n");
	fprintf(file, "$(PROGRAM):\t$(OBJS)\n");
	fprintf(file, "\tcc $(LDFLAGS) $(APPNAME) $(OBJS) $(XLIBS)\n\n");

	fprintf(file, "$(APPNAME).o:\t$(APPNAME).c\t$(APPNAME).h\n");
	fprintf(file, "\tcc -c $(CFLAGS) $(APPNAME).c\n\n");

	fprintf(file, "$(UTILNAME).o:\t$(UTILNAME).c\t$(UTILNAME).h\n");
	fprintf(file, "\tcc -c $(CFLAGS) $(UTILNAME).c\n\n");

	fprintf(file, "clean:\n");
	fprintf(file, "\trm -f *.o $(APPNAME)\n\n");

	fclose(file);

	return (OK);

}
/**************************************************
 *
 * Function: code.c
 *
 **************************************************/
int
code()
{
	int status;
	char *appname;
	
	appname = application_name;

	output_message("\nCoding files....");

	output_message("\nWriting Makefile");
	status = make_makefile(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	output_message("\nWriting application source file");
	status = make_appheader(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	status = make_appcfile(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

/* TBD
	output_message("\nWriting callbacks source file");
	status = make_callheader(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	status = make_callfile(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");
*/

	output_message("\nWriting application X defaults file");
	status = make_appdefs(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	output_message("\nWriting utilities source file");
	status = make_utilsrcfile(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	status = make_utilhdrfile(appname);
	if (status != OK)
		printf("Cannot overwrite existing file.\n");

	output_message("\nDone!");

	return (OK);

}
