/**************************************************
 * File: code_util.c - code utility functions 
 *
 *    Copyright (C) 1995  Bruce Parkin 
 *
 **************************************************/

/* include files */
#include <stdio.h>
#include <Xm/Xm.h>
#include "lxb.h"
#include "code.h"
#include "code_util.h"

/**************************************************
 *
 *	Fuction: make_utilhdrfile
 *
 **************************************************/
int
make_utilhdrfile(char *appname)
{
  int   status;
  FILE  *file;
  char  all[70];

  strcpy(all, appname);
  strcat(all, "_util.h");

  /* don't overwrite the old file */
  status = check_file(all);
  if (status != OK)
    return (NOTOK);

	/* create the utilities header file*/
	file = fopen(all, "w");
	if (file == NULL){
		perror("Problem with utilities headerfile");
		return (NOTOK);
	};

	fprintf(file, "/**************************************************\n");
	fprintf(file, "* File: %s_util.h - lxb utilities header file\n", appname);
	fprintf(file, "*\n");
	fprintf(file, "*    This file is generated by the program lxb.\n");
	fprintf(file, "*\n");  
	fprintf(file, "**************************************************/\n\n");
  fprintf(file, "Pixel\n");
  fprintf(file, "get_pixel(Widget w, char *resource_value);\n");
  fprintf(file, "XmFontList\n");
  fprintf(file, "get_fontlist(Widget w, char *fontname);\n");
	fclose(file);

	return (OK);
}
/**************************************************
 *
 *	Fuction: make_utilsrcfile
 *
 **************************************************/
int
make_utilsrcfile(char *appname)
{
  int   status;
  FILE  *file;
  char  all[70];

  strcpy(all, appname);
  strcat(all, "_util.c");

  /* don't overwrite the old file */
  status = check_file(all);
  if (status != OK)
    return (NOTOK);

	/* create the utilities source file*/
	file = fopen(all, "w");
	if (file == NULL){
		perror("Problem with utilities source file");
		return (NOTOK);
	};

	fprintf(file, "/**************************************************\n");
	fprintf(file, "* File: %s_util.c - lxb utilities source file\n", appname);
	fprintf(file, "*\n");
	fprintf(file, "*    This file is generated by the program lxb.\n");
	fprintf(file, "*\n");  
	fprintf(file, "**************************************************/\n\n");
	fprintf(file, "/* include files */\n");
	fprintf(file, "#include <stdio.h>\n");
	fprintf(file, "#include <Xm/Xm.h>\n\n");         
	fprintf(file, "#include \"%s_util.h\"\n\n", appname);         

  fprintf(file, "/**************************************************\n");
  fprintf(file, "* Function: get_pixel\n");
  fprintf(file, "**************************************************/\n");
  fprintf(file, "Pixel\n");
  fprintf(file, "get_pixel(Widget w, char *resource_value)\n");
  fprintf(file, "{\n"); 

  fprintf(file, "\tColormap colormap;\n"); 
  fprintf(file, "\tBoolean status;\n"); 
  fprintf(file, "\tXColor exact, color;\n"); 
  fprintf(file, "\n"); 

	fprintf(file, "\tcolormap = DefaultColormapOfScreen( DefaultScreenOfDisplay(XtDisplay(w)) );\n\n");
	fprintf(file, "\tstatus = XAllocNamedColor( XtDisplay(w), colormap, resource_value, &color, &exact);\n");
  fprintf(file, "\n"); 

	fprintf(file, "\tif (status == 0){\n");
	fprintf(file, "\t\tfprintf(stderr, \"Unknown color: %%s\", resource_value);\n");
	fprintf(file, "\t\tcolor.pixel = BlackPixelOfScreen( DefaultScreenOfDisplay(XtDisplay(w)) );\n");
	fprintf(file, "\t};\n");
  fprintf(file, "\n"); 

  fprintf(file, "\treturn (color.pixel);\n");
  fprintf(file, "}\n"); 

	/* Fontlist utility function */
  fprintf(file, "/**************************************************\n");
  fprintf(file, "* Function: get_fontlist\n");
  fprintf(file, "**************************************************/\n");

  fprintf(file, "XmFontList\n");
  fprintf(file, "get_fontlist(Widget w, char *fontname)\n");
  fprintf(file, "{\n"); 

  fprintf(file, "\tXFontStruct *font;\n"); 
  fprintf(file, "\tXmFontList fontlist;\n"); 
  fprintf(file, "\n"); 

	fprintf(file, "\tfont = XLoadQueryFont(XtDisplay(w), fontname);\n\n");

	fprintf(file, "\tif (font == NULL){\n");

	fprintf(file, "\t\tfprintf(stderr, \"\\tCould not load font: %%s\\n\\n\", fontname);\n");
	fprintf(file,"\t\tfprintf(stderr, \"\\tUsing this system's default font instead.\\n\\n\");\n");
	fprintf(file, "\t\tfont = XLoadQueryFont(XtDisplay(w), XtDefaultFont);\n");
	fprintf(file, "\t};\n\n");

	fprintf(file, "\tfontlist = XmFontListCreate(font, XmSTRING_DEFAULT_CHARSET);\n\n");


	fprintf(file, "\treturn (fontlist);\n");

	fprintf(file, "}\n");

	fclose(file);

	return (OK);
}
